package es.ucm.fdi.grasia.faerie.demos.teacherFinding.myLocationUpdater;

import es.ucm.fdi.grasia.faerie.core.*;
import es.ucm.fdi.grasia.faerie.demos.teacherFinding.model.Teacher;
import es.ucm.fdi.grasia.faerie.demos.teacherFinding.model.Timetable;
import es.ucm.fdi.grasia.faerie.demos.talkingAgents.model.MyLocation;
import java.util.TimerTask;
import java.util.Timer;
import java.util.Calendar;
import java.util.Properties;
import java.util.Map.Entry;
import java.net.URI;
import java.util.Map;
import es.ucm.fdi.grasia.faerie.core.model.BasicContextValue;
import es.ucm.fdi.grasia.faerie.core.util.Functions;
import static java.util.GregorianCalendar.*;
import org.apache.felix.scr.annotations.Component;

@Component(inherit = true)
@SuppressWarnings({ "unchecked", "rawtypes" })
public class MyLocationUpdater extends AbstractAttributeUpdaterComponent {

	@Override
	public int handle(ContextEvent event) {
		return Functions.contextElementAddedOrRemoved(MyLocation.class.getName(), event) ? ContextObserver.HANDLE_WRITE
				: ContextObserver.HANDLE_NONE;
	}

	@Override
	public void elementAdded(Attribute attributeInstance, ContextEvent event) {
		this.putUpdatedAttributeInstance(attributeInstance);

		Teacher entity = (Teacher) attributeInstance.getEntity();
		this.addAttributeInstanceLinkedElement(attributeInstance.getId(), new Timetable(entity));

		this.updateAttributeInstance(attributeInstance.getId());
	}

	private int parseDayOfWeek(String dayStr) {
		if (dayStr.equals("L"))
			return MONDAY;
		if (dayStr.equals("M"))
			return TUESDAY;
		if (dayStr.equals("X"))
			return WEDNESDAY;
		if (dayStr.equals("J"))
			return THURSDAY;
		if (dayStr.equals("V"))
			return FRIDAY;
		if (dayStr.equals("S"))
			return SATURDAY;
		if (dayStr.equals("D"))
			return SUNDAY;
		return -1;
	}

	private int dayGap(int today, int next) {
		return (next - today + 7) % 7;
	}

	@Override
	public void attributeInstanceLinkedElementsUpdated(final Attribute attributeInstance,
			Map<URI, ContextElement> linkedElements) {
		Timetable timetableAttInst = (Timetable) linkedElements.values().iterator().next();
		ContextValue<Properties> timetableValue = timetableAttInst.getValue();
		String classRoom = "Unknown";

		if (timetableValue != null) {
			Properties properties = timetableValue.getData();
			Calendar today = Calendar.getInstance();
			today.setFirstDayOfWeek(MONDAY);
			Calendar next = null;
			for (Entry<Object, Object> entry : properties.entrySet()) {
				String key = (String) entry.getKey();

				String[] timeFields = key.split("_");
				String[] startFields = timeFields[1].split("\\.");
				String[] endFields = timeFields[2].split("\\.");

				Calendar start = (Calendar) today.clone();
				start.add(DAY_OF_MONTH, dayGap(today.get(DAY_OF_WEEK), parseDayOfWeek(timeFields[0])));
				Calendar end = (Calendar) start.clone();

				start.set(HOUR_OF_DAY, Integer.parseInt(startFields[0]));
				start.set(MINUTE, Integer.parseInt(startFields[1]));
				start.set(SECOND, Integer.parseInt(startFields[2]));

				end.set(HOUR_OF_DAY, Integer.parseInt(endFields[0]));
				end.set(MINUTE, Integer.parseInt(endFields[1]));
				end.set(SECOND, Integer.parseInt(endFields[2]));

				if (today.after(start) && today.before(end)) {
					classRoom = (String) entry.getValue();
					next = end;
					break;
				}

				if (start.after(today) && (next == null || start.before(next)))
					next = start;
			}
			if (next != null) {
				next.add(SECOND, 2);
				new Timer("Update " + attributeInstance.getId(), true).schedule(new TimerTask() {

					@Override
					public void run() {
						MyLocationUpdater.this.updateAttributeInstance(attributeInstance.getId());
					}
				}, next.getTime());
			}
		}

		attributeInstance.update(new BasicContextValue(classRoom, 1.0, this.getBundleEntity()));
	}
}
